(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('@angular/animations'), require('@angular/core'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ngx-block-loading', ['exports', 'rxjs', 'rxjs/operators', '@angular/animations', '@angular/core', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-block-loading'] = {}, global.rxjs, global.rxjs.operators, global.ng.animations, global.ng.core, global.ng.common));
}(this, (function (exports, rxjs, operators, i1, i0, common) { 'use strict';

    function getArray(param) {
        var array = [];
        if (param) {
            if (Array.isArray(param)) {
                array = param;
            }
            else {
                array = [param];
            }
        }
        return array;
    }
    var loadingCount = 0;
    var isLoadingFullPage = new rxjs.BehaviorSubject(false);
    function updateFullPageLoading() {
        isLoadingFullPage.next(loadingCount > 0);
    }
    function ngxBlockLoading(params) {
        var loadingDirectives = getArray(params.blocking);
        return function (source) {
            return rxjs.defer(function () {
                loadingDirectives.forEach(function (d) { return d.start(); });
                if (params.fullPage) {
                    loadingCount++;
                    updateFullPageLoading();
                }
                return source.pipe(operators.finalize(function () {
                    loadingDirectives.forEach(function (d) { return d.stop(); });
                    if (params.fullPage) {
                        loadingCount--;
                        updateFullPageLoading();
                    }
                }));
            });
        };
    }

    var AnimationHelperService = /** @class */ (function () {
        function AnimationHelperService(animationBuilder) {
            this.animationBuilder = animationBuilder;
            this.hasLoadingElement = [];
            this.players = [];
            this.loadingElements = [];
        }
        AnimationHelperService.prototype.elementRefEquals = function (one, two) {
            return one === null || one === void 0 ? void 0 : one.nativeElement.isSameNode(two === null || two === void 0 ? void 0 : two.nativeElement);
        };
        AnimationHelperService.prototype.getPlayerWrapperIndex = function (element) {
            var _this = this;
            var playerWrapperIndex = this.players.findIndex(function (e) { return _this.elementRefEquals(e.element, element); });
            return playerWrapperIndex;
        };
        AnimationHelperService.prototype.addPlayer = function (element, player) {
            if (!element) {
                return;
            }
            var playerWrapperIndex = this.getPlayerWrapperIndex(element);
            if (playerWrapperIndex === -1) {
                this.players.push({ element: element, players: [player] });
                return;
            }
            var playerWrapper = this.players[playerWrapperIndex];
            this.players[playerWrapperIndex] = {
                element: element,
                players: playerWrapper.players.concat(player)
            };
        };
        AnimationHelperService.prototype.getPlayers = function (element) {
            var playerWrapperIndex = this.getPlayerWrapperIndex(element);
            if (playerWrapperIndex === -1) {
                return [];
            }
            var players = this.players[playerWrapperIndex].players;
            return players;
        };
        AnimationHelperService.prototype.clearPlayers = function (element) {
            var playerWrapperIndex = this.getPlayerWrapperIndex(element);
            if (playerWrapperIndex === -1) {
                return;
            }
            this.players.splice(playerWrapperIndex, 1);
        };
        AnimationHelperService.prototype.getLoadingElementIndex = function (element) {
            var _this = this;
            var loadingElementIndex = this.loadingElements.findIndex(function (e) { return _this.elementRefEquals(e.element, element); });
            return loadingElementIndex;
        };
        AnimationHelperService.prototype.addLoadingElement = function (element, loadingElement) {
            if (!element) {
                return;
            }
            var loadingElementIndex = this.getLoadingElementIndex(element);
            if (loadingElementIndex === -1) {
                this.loadingElements.push({
                    element: element,
                    loadingElement: loadingElement
                });
            }
        };
        AnimationHelperService.prototype.getLoadingElement = function (element) {
            if (!element) {
                return;
            }
            var loadingElementIndex = this.getLoadingElementIndex(element);
            if (loadingElementIndex === -1) {
                return undefined;
            }
            var loadingElement = this.loadingElements[loadingElementIndex].loadingElement;
            if (!loadingElement.nativeElement.parentNode) {
                loadingElement.nativeElement.parentNode = element.nativeElement;
            }
            return loadingElement;
        };
        AnimationHelperService.prototype.removeLoadingElement = function (element) {
            var loadingElementIndex = this.getLoadingElementIndex(element);
            if (loadingElementIndex === -1) {
                return;
            }
            this.loadingElements.splice(loadingElementIndex, 1);
        };
        AnimationHelperService.prototype.isElementLoading = function (element) {
            var _this = this;
            if (!element) {
                return false;
            }
            return this.hasLoadingElement.some(function (e) { return _this.elementRefEquals(e, element); });
        };
        AnimationHelperService.prototype.runPlayer = function (element, metadata, destroyOnDone, onDone) {
            var _this = this;
            if (destroyOnDone === void 0) { destroyOnDone = false; }
            var factory = this.animationBuilder.build(i1.useAnimation(metadata));
            var player = factory.create(element.nativeElement);
            var playerWrapper = {
                player: player,
                done: false
            };
            player.onDone(function () {
                playerWrapper.done = true;
                var players = _this.getPlayers(element);
                if (destroyOnDone) {
                    players.forEach(function (animationPlayer) {
                        _this.tryRunMethodOnPlayer(function () { return animationPlayer.player.destroy(); });
                    });
                }
                if (players.every(function (player) { return player.done; })) {
                    _this.clearPlayers(element);
                }
                if (onDone) {
                    onDone();
                }
            });
            player.play();
            this.addPlayer(element, playerWrapper);
        };
        AnimationHelperService.prototype.isAnimationPlayerDone = function (player) {
            return (player.done ||
                player.player.totalTime === player.player.getPosition());
        };
        AnimationHelperService.prototype.tryRunMethodOnPlayer = function (playerMethod, callbackIfFailed) {
            try {
                playerMethod();
            }
            catch (error) {
                // This error means that the player has been destroyed
                // However, Angular doesn't provide a means of checking if this has been done already
                // So we have to catch the error
                if (error &&
                    error.message &&
                    !error.message.includes('Unable to find the timeline player referenced by')) {
                    throw error;
                }
                if (callbackIfFailed) {
                    callbackIfFailed();
                }
            }
        };
        AnimationHelperService.prototype.animate = function (element, metadata, destroyOnDone, onDone) {
            var _this = this;
            if (destroyOnDone === void 0) { destroyOnDone = false; }
            if (element) {
                var players = this.getPlayers(element);
                var notDonePlayers = players.filter(function (player) { return !_this.isAnimationPlayerDone(player); });
                if (notDonePlayers.length > 0) {
                    var lastPlayerWrapper_1 = notDonePlayers[notDonePlayers.length - 1];
                    this.tryRunMethodOnPlayer(function () { return lastPlayerWrapper_1.player.onDone(function () {
                        _this.runPlayer(element, metadata, destroyOnDone, onDone);
                    }); }, function () {
                        _this.runPlayer(element, metadata, destroyOnDone, onDone);
                    });
                }
                else {
                    this.runPlayer(element, metadata, destroyOnDone, onDone);
                }
            }
        };
        AnimationHelperService.prototype.tryCreateLoadingElement = function (element, loadingElement, classes, styles, inTime, renderer, addLoadingClass) {
            if (!element || this.isElementLoading(element)) {
                return;
            }
            renderer.addClass(element.nativeElement, classes.container);
            renderer.appendChild(element.nativeElement, loadingElement.nativeElement);
            if (addLoadingClass) {
                renderer.addClass(loadingElement.nativeElement, classes.loading);
            }
            this.addLoadingElement(element, loadingElement);
            this.hasLoadingElement.push(element);
            this.animate(element, i1.animation([
                styles.containerNotLoading,
                i1.animate(inTime, styles.containerLoading)
            ]));
        };
        AnimationHelperService.prototype.tryRemoveLoadingElement = function (element, classes, styles, times, renderer) {
            var _this = this;
            if (!element || !this.isElementLoading(element)) {
                return;
            }
            var loadingElement = this.getLoadingElement(element);
            var players = this.getPlayers(element);
            players.forEach(function (player) {
                if (!_this.isAnimationPlayerDone(player)) {
                    _this.tryRunMethodOnPlayer(function () { return player.player.destroy(); });
                }
            });
            this.animate(element, i1.animation([
                styles.loadingContainer,
                i1.animate(times.outTime, i1.style({
                    height: element.nativeElement.scrollHeight
                }))
            ]), true, function () {
                var hasLoadingElementIndex = _this.hasLoadingElement.findIndex(function (e) { return _this.elementRefEquals(e, element); });
                if (hasLoadingElementIndex > -1) {
                    _this.hasLoadingElement.splice(hasLoadingElementIndex, 1);
                }
                _this.animate(loadingElement, i1.animation([
                    styles.loading,
                    i1.animate(times.loaderOutTime, styles.notLoading)
                ]), true, function () {
                    if (loadingElement) {
                        renderer.removeChild(element.nativeElement, loadingElement.nativeElement);
                        _this.removeLoadingElement(element);
                    }
                    else {
                        throw new Error('There is no element that was created for blocking, something has gone wrong.');
                    }
                    renderer.removeClass(element.nativeElement, classes.container);
                });
            });
        };
        return AnimationHelperService;
    }());
    AnimationHelperService.ɵprov = i0.ɵɵdefineInjectable({ factory: function AnimationHelperService_Factory() { return new AnimationHelperService(i0.ɵɵinject(i1.AnimationBuilder)); }, token: AnimationHelperService, providedIn: "root" });
    AnimationHelperService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    AnimationHelperService.ctorParameters = function () { return [
        { type: i1.AnimationBuilder }
    ]; };

    var NGX_BLOCK_LOADING_OPTIONS = new i0.InjectionToken('NGX_BLOCK_LOADING_OPTIONS');
    var DEFAULT_OPTIONS = {
        inTime: '0.25s',
        outTime: '0.25s',
        loaderOutTime: '0.25s',
        containerHeight: '100px',
        loadingContainerClass: 'ngx-block-loading--container',
        loadingClass: 'ngx-block-loading',
        loadingFullPageClass: 'ngx-block-loading__full-page'
    };

    var NgxBlockLoadingFullPageComponent = /** @class */ (function () {
        function NgxBlockLoadingFullPageComponent(options, animationHelper) {
            var _this = this;
            this.options = options;
            this.animationHelper = animationHelper;
            this.loading = false;
            this.onDestroy$ = new rxjs.Subject();
            this.loaderOutTime = this.options.loaderOutTime;
            this.loadingClass = this.options.loadingClass;
            this.loadingFullPageClass = this.options.loadingFullPageClass;
            isLoadingFullPage
                .pipe(operators.takeUntil(this.onDestroy$))
                .subscribe(function (loading) {
                if (_this.isLoading == undefined) {
                    _this.updateLoading(loading);
                }
            });
        }
        Object.defineProperty(NgxBlockLoadingFullPageComponent.prototype, "loadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'visible',
                    opacity: 1
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingFullPageComponent.prototype, "notLoadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'hidden',
                    opacity: 0
                });
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockLoadingFullPageComponent.prototype.ngOnChanges = function (changes) {
            var loadingChanges = changes['loading'];
            if (loadingChanges) {
                this.updateLoading(loadingChanges.currentValue);
            }
        };
        NgxBlockLoadingFullPageComponent.prototype.updateLoading = function (loading) {
            if (loading) {
                this.loading = true;
            }
            else {
                this.removeFullPageLoading();
            }
        };
        NgxBlockLoadingFullPageComponent.prototype.removeFullPageLoading = function () {
            var _this = this;
            this.animationHelper.animate(this.element, i1.animation([
                this.loadingStyle,
                i1.animate(this.loaderOutTime, this.notLoadingStyle)
            ]), true, function () {
                _this.loading = false;
            });
        };
        return NgxBlockLoadingFullPageComponent;
    }());
    NgxBlockLoadingFullPageComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'ngx-block-loading-full-page',
                    template: "<div #element>\n  <ng-container *ngIf=\"loading && template\">\n    <ng-container *ngTemplateOutlet=\"template\"></ng-container>\n  </ng-container>\n\n  <div class=\"{{loadingClass}} {{loadingFullPageClass}}\" *ngIf=\"loading && !template\"></div>\n</div>\n",
                    styles: [""]
                },] }
    ];
    NgxBlockLoadingFullPageComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: AnimationHelperService }
    ]; };
    NgxBlockLoadingFullPageComponent.propDecorators = {
        template: [{ type: i0.Input }],
        loaderOutTime: [{ type: i0.Input }],
        loadingClass: [{ type: i0.Input }],
        loadingFullPageClass: [{ type: i0.Input }],
        isLoading: [{ type: i0.Input, args: ['loading',] }],
        element: [{ type: i0.ViewChild, args: ['element', { static: false },] }]
    };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2)
            for (var i = 0, l = from.length, ar; i < l; i++) {
                if (ar || !(i in from)) {
                    if (!ar)
                        ar = Array.prototype.slice.call(from, 0, i);
                    ar[i] = from[i];
                }
            }
        return to.concat(ar || Array.prototype.slice.call(from));
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, state, kind, f) {
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    }
    function __classPrivateFieldSet(receiver, state, value, kind, f) {
        if (kind === "m")
            throw new TypeError("Private method is not writable");
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    }

    var BaseLoadingDirective = /** @class */ (function () {
        function BaseLoadingDirective(element, viewContainerRef, renderer, options, animationHelper) {
            this.element = element;
            this.viewContainerRef = viewContainerRef;
            this.renderer = renderer;
            this.options = options;
            this.animationHelper = animationHelper;
            this.onDestroy$ = new rxjs.Subject();
            this.inTime = this.options.inTime;
            this.outTime = this.options.outTime;
            this.loaderOutTime = this.options.loaderOutTime;
            this.containerHeight = this.options.containerHeight;
            this.loadingContainerClass = this.options.loadingContainerClass;
            this.loadingClass = this.options.loadingClass;
        }
        Object.defineProperty(BaseLoadingDirective.prototype, "loadingContainerStyle", {
            get: function () {
                return i1.style({
                    height: this.containerHeight
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(BaseLoadingDirective.prototype, "loadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'visible',
                    opacity: 1
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(BaseLoadingDirective.prototype, "notLoadingContainerStyle", {
            get: function () {
                var _a, _b;
                return i1.style({
                    height: ((_b = (_a = this.element) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.scrollHeight) || i1.AUTO_STYLE
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(BaseLoadingDirective.prototype, "notLoadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'hidden',
                    opacity: 0
                });
            },
            enumerable: false,
            configurable: true
        });
        BaseLoadingDirective.prototype.ngOnDestroy = function () {
            this.onDestroy$.next();
            this.onDestroy$.complete();
        };
        BaseLoadingDirective.prototype.start = function () {
            this.updateLoadingElement(true);
        };
        BaseLoadingDirective.prototype.stop = function () {
            this.updateLoadingElement(false);
        };
        BaseLoadingDirective.prototype.updateLoadingElement = function (isLoading) {
            var _this = this;
            if (isLoading) {
                this.animationHelper.tryCreateLoadingElement(this.elementToLoad, this.getLoadingElement(), {
                    loading: this.loadingClass,
                    container: this.loadingContainerClass
                }, {
                    containerLoading: this.loadingContainerStyle,
                    containerNotLoading: this.notLoadingContainerStyle
                }, this.inTime, this.renderer, this.template == undefined);
            }
            else {
                // Waiting a tick here because we need to let Angular finish rendering
                // It is likely that it actually hasn't
                setTimeout(function () { return _this.animationHelper.tryRemoveLoadingElement(_this.elementToLoad, {
                    container: _this.loadingContainerClass
                }, {
                    loading: _this.loadingStyle,
                    loadingContainer: _this.loadingContainerStyle,
                    notLoading: _this.notLoadingStyle
                }, {
                    outTime: _this.outTime,
                    loaderOutTime: _this.loaderOutTime
                }, _this.renderer); }, 0);
            }
        };
        BaseLoadingDirective.prototype.getTemplateElement = function () {
            this.viewContainerRef.clear();
            var viewRef = this.viewContainerRef.createEmbeddedView(this.template);
            var element = viewRef.rootNodes[0];
            var duplicatedElement = element.cloneNode(true);
            this.viewContainerRef.clear();
            return new i0.ElementRef(duplicatedElement);
        };
        BaseLoadingDirective.prototype.getLoadingElement = function () {
            var loadingElement;
            if (this.template) {
                loadingElement = this.getTemplateElement();
            }
            else {
                loadingElement = new i0.ElementRef(this.renderer.createElement('div'));
            }
            return loadingElement;
        };
        return BaseLoadingDirective;
    }());
    BaseLoadingDirective.decorators = [
        { type: i0.Directive }
    ];
    BaseLoadingDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: i0.ViewContainerRef },
        { type: i0.Renderer2 },
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: AnimationHelperService }
    ]; };
    BaseLoadingDirective.propDecorators = {
        inTime: [{ type: i0.Input }],
        outTime: [{ type: i0.Input }],
        loaderOutTime: [{ type: i0.Input }],
        containerHeight: [{ type: i0.Input }],
        loadingContainerClass: [{ type: i0.Input }],
        loadingClass: [{ type: i0.Input }],
        template: [{ type: i0.Input }]
    };

    var NgxBlockLoadingDirective = /** @class */ (function (_super) {
        __extends(NgxBlockLoadingDirective, _super);
        function NgxBlockLoadingDirective(element, viewContainerRef, renderer, options, animationHelper) {
            var _this = _super.call(this, element, viewContainerRef, renderer, options, animationHelper) || this;
            _this._isLoading = false;
            _this.renderingDirectives = [];
            return _this;
        }
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "isLoading", {
            get: function () {
                return this._isLoading;
            },
            set: function (value) {
                this._isLoading = value !== '' || value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "elementToLoad", {
            get: function () {
                return this.element;
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockLoadingDirective.prototype.ngOnChanges = function (changes) {
            var isLoadingChanges = changes['isLoading'];
            if (isLoadingChanges) {
                this.updateLoadingElement(isLoadingChanges.currentValue);
            }
        };
        NgxBlockLoadingDirective.prototype.getRenderingElementIndex = function (renderingDirective) {
            return this.renderingDirectives.findIndex(function (r) {
                var _a, _b;
                return (_a = r.elementToLoad) === null || _a === void 0 ? void 0 : _a.nativeElement.isSameNode((_b = renderingDirective.elementToLoad) === null || _b === void 0 ? void 0 : _b.nativeElement);
            });
        };
        NgxBlockLoadingDirective.prototype.addRenderingElement = function (renderingDirective) {
            var _a;
            if (((_a = renderingDirective.elementToLoad) === null || _a === void 0 ? void 0 : _a.nativeElement) == undefined) {
                return;
            }
            var renderingElementIndex = this.getRenderingElementIndex(renderingDirective);
            if (renderingElementIndex === -1) {
                this.renderingDirectives.push(renderingDirective);
                if (!this.animationHelper.isElementLoading(this.elementToLoad)) {
                    this.start();
                }
            }
        };
        NgxBlockLoadingDirective.prototype.removeRenderingElement = function (renderingDirective) {
            var renderingElementIndex = this.getRenderingElementIndex(renderingDirective);
            if (renderingElementIndex > -1) {
                this.renderingDirectives.splice(renderingElementIndex, 1);
            }
        };
        NgxBlockLoadingDirective.prototype.stop = function () {
            if (this.renderingDirectives.length === 0) {
                _super.prototype.stop.call(this);
            }
        };
        return NgxBlockLoadingDirective;
    }(BaseLoadingDirective));
    NgxBlockLoadingDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[ngxBlockLoading]',
                    exportAs: 'ngxBlockLoading'
                },] }
    ];
    NgxBlockLoadingDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: i0.ViewContainerRef },
        { type: i0.Renderer2 },
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: AnimationHelperService }
    ]; };
    NgxBlockLoadingDirective.propDecorators = {
        isLoading: [{ type: i0.Input, args: ['ngxBlockLoading',] }]
    };

    var NgxBlockRenderingDirective = /** @class */ (function (_super) {
        __extends(NgxBlockRenderingDirective, _super);
        function NgxBlockRenderingDirective() {
            var _this = _super.apply(this, __spreadArray([], __read(arguments))) || this;
            _this._isRendering = true;
            _this.startOnInit = true;
            _this.forceStart = false;
            return _this;
        }
        Object.defineProperty(NgxBlockRenderingDirective.prototype, "isRendering", {
            get: function () {
                return this._isRendering;
            },
            set: function (value) {
                this._isRendering = value !== '' && value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockRenderingDirective.prototype, "elementToLoad", {
            get: function () {
                var _a, _b;
                if ((_a = this.element) === null || _a === void 0 ? void 0 : _a.nativeElement.parentElement) {
                    return new i0.ElementRef((_b = this.element) === null || _b === void 0 ? void 0 : _b.nativeElement.parentElement);
                }
                return undefined;
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockRenderingDirective.prototype.ngOnInit = function () {
            if (this.startOnInit && this.isRendering && !this.loadingDirective) {
                this.start();
            }
            if (this.loadingDirective && this.isRendering) {
                this.loadingDirective.addRenderingElement(this);
            }
        };
        NgxBlockRenderingDirective.prototype.ngOnChanges = function (changes) {
            if (changes['isRendering'] && !this.isRendering) {
                if (this.loadingDirective) {
                    this.loadingDirective.removeRenderingElement(this);
                    this.loadingDirective.stop();
                    return;
                }
                this.stop();
            }
            if (changes['forceStart'] && this.isRendering) {
                this.start();
            }
        };
        return NgxBlockRenderingDirective;
    }(BaseLoadingDirective));
    NgxBlockRenderingDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[ngxBlockRendering]'
                },] }
    ];
    NgxBlockRenderingDirective.propDecorators = {
        isRendering: [{ type: i0.Input, args: ['ngxBlockRendering',] }],
        startOnInit: [{ type: i0.Input }],
        forceStart: [{ type: i0.Input }],
        loadingDirective: [{ type: i0.Input }]
    };

    var NgxBlockLoadingModule = /** @class */ (function () {
        function NgxBlockLoadingModule() {
        }
        NgxBlockLoadingModule.forRoot = function (options) {
            return {
                ngModule: NgxBlockLoadingModule,
                providers: [
                    (options === null || options === void 0 ? void 0 : options.provider) || {
                        provide: NGX_BLOCK_LOADING_OPTIONS,
                        useValue: setDefaultOptions(options === null || options === void 0 ? void 0 : options.config)
                    }
                ]
            };
        };
        return NgxBlockLoadingModule;
    }());
    NgxBlockLoadingModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [common.CommonModule],
                    declarations: [
                        NgxBlockLoadingDirective,
                        NgxBlockRenderingDirective,
                        NgxBlockLoadingFullPageComponent
                    ],
                    exports: [
                        NgxBlockLoadingDirective,
                        NgxBlockRenderingDirective,
                        NgxBlockLoadingFullPageComponent
                    ]
                },] }
    ];
    function setDefaultOptions(options) {
        var defaultOptions = DEFAULT_OPTIONS;
        return {
            loadingTemplate: (options === null || options === void 0 ? void 0 : options.loadingTemplate) || defaultOptions.loadingTemplate,
            inTime: (options === null || options === void 0 ? void 0 : options.inTime) || defaultOptions.inTime,
            outTime: (options === null || options === void 0 ? void 0 : options.outTime) || defaultOptions.outTime,
            loaderOutTime: (options === null || options === void 0 ? void 0 : options.loaderOutTime) || defaultOptions.loaderOutTime,
            containerHeight: (options === null || options === void 0 ? void 0 : options.containerHeight) || defaultOptions.containerHeight,
            loadingContainerClass: (options === null || options === void 0 ? void 0 : options.loadingContainerClass) ||
                defaultOptions.loadingContainerClass,
            loadingClass: (options === null || options === void 0 ? void 0 : options.loadingClass) || defaultOptions.loadingClass,
            loadingFullPageClass: (options === null || options === void 0 ? void 0 : options.loadingFullPageClass) || defaultOptions.loadingFullPageClass
        };
    }

    /*
     * Public API Surface of loading
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NGX_BLOCK_LOADING_OPTIONS = NGX_BLOCK_LOADING_OPTIONS;
    exports.NgxBlockLoadingDirective = NgxBlockLoadingDirective;
    exports.NgxBlockLoadingFullPageComponent = NgxBlockLoadingFullPageComponent;
    exports.NgxBlockLoadingModule = NgxBlockLoadingModule;
    exports.NgxBlockRenderingDirective = NgxBlockRenderingDirective;
    exports.ngxBlockLoading = ngxBlockLoading;
    exports.ɵa = AnimationHelperService;
    exports.ɵb = BaseLoadingDirective;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-block-loading.umd.js.map
